#include "IR_Decoder.h"

std::list<IR_Decoder*>& IR_Decoder::get_dec_list() // определение функции
{
    static std::list<IR_Decoder*> dec_list; // статическая локальная переменная
    return dec_list; // возвращается ссылка на переменную
}

IR_Decoder::IR_Decoder(const uint8_t isrPin, uint16_t addr, IR_Encoder *encPair)
    : IR_DecoderRaw(isrPin, addr, encPair)
{
    get_dec_list().push_back(this);
};

IR_Decoder::~IR_Decoder()
{
    IR_Decoder::get_dec_list().remove(this);
}

void IR_Decoder::tick()
{
    for (const auto &element : IR_Decoder::get_dec_list())
    {
        element->_tick();
    }
}

void IR_Decoder::_tick()
{
    IR_DecoderRaw::tick();
    if (availableRaw())
    {
#ifdef IRDEBUG_INFO
        Serial.println("PARSING RAW DATA");
#endif
        isWaitingAcceptSend = false;
        switch (packInfo.buffer[0] >> 5 & IR_MASK_MSG_TYPE)
        {
        case IR_MSG_DATA_ACCEPT:
        case IR_MSG_DATA_NOACCEPT:
            gotData.set(&packInfo, id);
            break;
        case IR_MSG_BACK:
        case IR_MSG_BACK_TO:
            gotBackData.set(&packInfo, id);
            break;
        case IR_MSG_REQUEST:
            gotRequest.set(&packInfo, id);
            break;
        case IR_MSG_ACCEPT:
            gotAccept.set(&packInfo, id);
            break;

        default:
            break;
        }
        if (gotData.isAvailable && (gotData.getMsgType() == IR_MSG_DATA_ACCEPT))
        {
            acceptSendTimer = millis();
            addrAcceptSendTo = gotData.getAddrFrom();
            acceptCustomByte = crc8(gotData.getDataPrt(), 0, gotData.getDataSize(), poly1);
            if (addrAcceptSendTo && addrAcceptSendTo < IR_Broadcast)
                isWaitingAcceptSend = true;
        }
        gotRaw.set(&packInfo, id);
    }
    if (isWaitingAcceptSend && millis() - acceptSendTimer > 75)
    {
        encoder->sendAccept(addrAcceptSendTo, acceptCustomByte);
        isWaitingAcceptSend = false;
    }
}